%==============================
%% HH Comparison of all Methods
%==============================
%% Forward Euler Method
clc; clear;

%Variables - to change according to the particular assignment questions
I=0.1; %External Current Applied  I=0.1
gbarNa=1.2; % mS/cm^2 Na conductance gbarNa=1.2
gbarK= 0; % mS/cm^2 K conductance gbarK=0.36
gbarl=0.003; % mS/cm^2 Leakage conductance gbarl=0.003

%Constants  - these stay the same - DO NOT CHANGE
Cm=0.01; % Membrane Capcitance uF/cm^2  Cm=0.01
ENa=55.17; % mv Na reversal potential ENa=55.17
EK=-72.14; % mv K reversal potential EK=-72.14
El=-49.42; % mv Leakage reversal potential El=-49.42


dt=0.04; % Time Step ms
t=0:dt:30; %Time Array ms

V(1)=-60; % Initial Membrane voltage
m(1)=am(V(1))/(am(V(1))+bm(V(1))); % Initial m-value
n(1)=an(V(1))/(an(V(1))+bn(V(1))); % Initial n-value
h(1)=ah(V(1))/(ah(V(1))+bh(V(1))); % Initial h-value
for i=1:length(t)-1
    %Euler method to find the next m/n/h value
    m(i+1)=m(i)+dt*((am(V(i))*(1-m(i)))-(bm(V(i))*m(i)));
    n(i+1)=n(i)+dt*((an(V(i))*(1-n(i)))-(bn(V(i))*n(i)));
    h(i+1)=h(i)+dt*((ah(V(i))*(1-h(i)))-(bh(V(i))*h(i)));
    gNa=gbarNa*m(i)^3*h(i);
    gK=gbarK*n(i)^4;
    gl=gbarl;
    INa=gNa*(V(i)-ENa);
    IK=gK*(V(i)-EK);
    Il=gl*(V(i)-El);
    %Euler method to find the next voltage value
    V(i+1)=V(i)+(dt)*((1/Cm)*(I-(INa+IK+Il)));
end
%Store variables for graphing later
FE=V;
FEm=m;
FEn=n;
FEh=h;
clear V m n h;

%% Runge-Kutta Method
V(1)=-60; % Initial Membrane voltage
m(1)=am(V(1))/(am(V(1))+bm(V(1))); % Initial m-value
n(1)=an(V(1))/(an(V(1))+bn(V(1))); % Initial n-value
h(1)=ah(V(1))/(ah(V(1))+bh(V(1))); % Initial h-value


%% Plots
%Plot the functions
%h=figure;
j=axes; set(j,'FontSize',20);
plot(t,FE,'LineWidth',3);
axis([t(1) t(end) -100 100]);
k=legend('Membrane Potential'); set(k,'FontSize',20);
l=xlabel('Time (ms)','FontSize',20);
m=ylabel('Voltage (mV)','FontSize',20);
n=title(['Voltage Change for Input level:', num2str(I)],'FontSize',20);
